c________________________________________________________________________
c
c*****wavein
c
c          This subroutine reads the tabulated deuteron wave function
c       components from nunit. The wave function is normalized such that
c       it is in units of (MeV)^{-3/2}. The wave function table is then
c       reorganized in a form more convenient for the expression of the
c       Gross wave functions in the helicity basis.
c
      subroutine wavein(nunit,ndata,q,wt,zdata,paramfile,ipwavea,inorma)
c
      implicit real*8(a-h,o-z)
c
      parameter(nwave=4,mndata=64)
      parameter(pi=3.141592653589793,twopi3=(2.*pi)**3)
c
      dimension q(mndata),zdata(mndata,nwave),wt(mndata)
c
      common/wavpar/ipwave,inorm
c
      character*25 paramfile
c
 1    format(a25)
c
      ipwave=ipwavea
	inorm=inorma
      read(nunit,1) paramfile
cgpg      write(6,*) paramfile
cgpg      print *
      read(nunit,*) ndata
      do i=1,ndata
        read(nunit,*) q(i),(zdata(i,j),j=1,nwave),wt(i)
c	  write(6,*) q(i),(zdata(i,j),j=1,nwave)
      end do
c
      return
      end
c
c_________________________________________________________________________
c
c*****wave_spline_init
c
c           This subroutine sets up a cubic spline fit to the wave 
c        function components as a function of the natural log of the
c        magnitude of the momentum. The routine DEEP_SPLINE is called to
c        initialize the spline parameters for this fit.
c
c
c     INPUT
c          aq..................vector of gauss points
c          s...................vector of gauss weights
c          zdata...............matrix containing normalized wave functions
c                              in (GeV)^{-3/2}
c          n...................number of gauss points
c          w...................bound state mass in MeV
c          wn..................nucleon mass in MeV
c
c
      subroutine wave_spline_init(aq,s,zdata,n,w,wn)
c
      implicit real*8(a-h,o-z)
c
      parameter(nwave=4,mndata=64)
c
      common/wavpar/ipwave,inorm
c
      common/wavpar2/q(mndata),z(mndata,nwave),z2(mndata,nwave),
     &              wt(mndata),ndata
c
      common/xtrap/xpar(nwave,2,2),ipup,lwave(nwave)
c
      dimension aq(mndata),s(mndata),zdata(mndata,nwave)
c
      parameter(pi=3.141592653589793,twopi3=(2.*pi)**3)
c
      real*8 mn,mt
      common/dpara/mn,mt
c
      anorm=sqrt(twopi3*1.d-9)
c
      lwave(1)=0
      lwave(2)=2
      lwave(3)=1
      lwave(4)=1
c
      yp=1.e30
c
      ndata=n
      mn=wn
      mt=w
c
      if(ipwave.eq.1)then
        nwavep=nwave
	else
	  nwavep=2
	endif    
      sum=0.0
      do i=1,ndata
        q(i)=aq(i)
        wt(i)=s(i)
        do j=1,nwavep
          z(i,j)=zdata(i,j)*anorm ! wave functions in (MeV)^{-3/2} and
                                  ! factor of (2*pi)^{3/2} included for
                                  ! consistent treatment of phase space
                                  ! integrals
		sum=sum+wt(i)*q(i)**2*z(i,j)**2 
        end do
      end do
	sum=sum/twopi3
cgpg	print *,' sum=',sum
	if(inorm.eq.1) then
	  do i=1,ndata
          q(i)=aq(i)
          wt(i)=s(i)
          do j=1,nwavep
            z(i,j)=z(i,j)/sqrt(sum)
	    end do
	  end do
	end if
c
c           set up spline parameters of the wave function array
c
      call deep_spline(q,z,ndata,nwave,mndata,nwave,yp,yp,z2)
c
c           find parameters for fit to low p such that each
c           wave function goes as p**l
c
      ilow=1
      iup=2
      do j=1,nwave
        xpar(j,1,1)=(z(iup,j)/q(iup)**lwave(j)
     &              -z(ilow,j)/q(ilow)**lwave(j))
     &              /(q(iup)-q(ilow))
        xpar(j,2,1)=(q(iup)*z(ilow,j)/q(ilow)**lwave(j)
     &              -q(ilow)*z(iup,j)/q(iup)**lwave(j))
     &              /(q(iup)-q(ilow))
      end do      
c
      return
      end
c
c__________________________________________________________________________
c
c*****wav
c
c          This subroutine returns the for wave function components in
c       the array zp as interpolated for a given in input momentum p.
c       p is assumed to be in MeV. The cubic spline routine DEEP_SPLINT performs
c       the interpolation.
c
c
c           INPUT
c                   p............magnitude of momentum in MeV
c
c           OUTPUT
c                   zp(i)........deuteron wave functions
c
c                   zp(1)= s wave
c                   zp(2)= d wave
c                   zp(3)= p wave triplet
c                   zp(4)= p wave singlet
c
c
      subroutine wav(p,zp)
c
      implicit real*8(a-h,o-z)
c
      parameter(nwave=4,mndata=64)
c
      common/wavpar/ipwave,inorm
c
      common/wavpar2/q(mndata),z(mndata,nwave),z2(mndata,nwave),
     &              wt(mndata),ndata
c
      common/xtrap/xpar(nwave,2,2),ipup,lwave(nwave)
c
      dimension zp(nwave)
c
      if(p.lt.q(1)) then
c
        zp(1)=xpar(1,1,1)*p+xpar(1,2,1)
        do i=2,nwave
          zp(i)=p**lwave(i)*(xpar(i,1,1)*p+xpar(i,2,1))
        end do
c
      else if(p.gt.q(ndata)) then
c
        do i=1,nwave
          zp(i)=0.d0
        end do
c
      else
c       
        call deep_splint(q,z,z2,ndata,nwave,mndata,nwave,p,zp)
      endif
c
      if(ipwave.eq.0) then
	  zp(3)=0.0d0
	  zp(4)=0.0d0
	end if
c
c      zp(1)=0.0
c       zp(2)=0.0
c       zp(3)=0.0
c       zp(4)=0.0
c      zp(4)=-zp(4)
c
      return
      end
c
c______________________________________________________________________
c
c*****deuwav
c
c         This subroutine constructs the Gross deuteron wave functions.
c
c       INPUT
c           p.............magnitude of momentum in MeV
c           irel=1........relativistic wave functions
c                0........nonrelativistic wave functions
c
c       OUTPUT
c           a(lnuc,lnucp,irho,iwave)...array containing deuteron wave
c                                      functions
c
c             lnuc=0.........positive nucleon helicity
c                 =1.........negative nucleon helicity
c             lnucp=0........positive nucleon helicity
c                  =1........negative nucleon helicity
c             ienrgy=0.......positive energy projections
c                    1.......negative energy projections
c             iwave=0........wave functions
c                   2........vertex functions
c
c
      subroutine deuwav(p,irel,a)
c
      implicit real*8(a-h,o-z)
c
      parameter(nwave=4,mndata=64)
c
      common/wavpar2/q(mndata),z(mndata,nwave),z2(mndata,nwave),
     &              wt(mndata),ndata
c
      dimension zp(nwave),a(0:1,0:1,0:1,0:1)
c
      real*8 mn,mt
      common/dpara/mn,mt
c
      parameter(root2=1.414213562373095,root3=1.732050807568877)
      parameter(pi=3.141592653589793)
c
c           obtain the wave function components and the Dirac spinors
c
      fac=1.d0/sqrt(8.d0*pi)
c
      call wav(p,zp)
c
      ep=sqrt(p**2+mn**2)
c
      g_plus_inv=ep*(2.d0*ep-mt)/mn
      g_minus_inv=-ep*mt/mn
c
      gamma1=g_plus_inv*(zp(1)+root2*zp(2))
      gamma2=-g_minus_inv*root3*zp(4)
      gamma3=g_plus_inv*(root2*zp(1)-zp(2))
      gamma4=-g_minus_inv*root3*zp(3)
c
c              construct new combinations of wave matrix
c
c      if(irel.eq.1) then               ! relativistic wave functions
        pomn=p/mn
        epomn=ep/mn
c
        a(0,0,0,0)=fac*(zp(1)+root2*zp(2)+root3*pomn*zp(4))
        a(1,1,0,0)=a(0,0,0,0)
        a(1,0,0,0)=fac*(root2*zp(1)-zp(2)-root3*pomn*zp(3))
        a(0,1,0,0)=a(1,0,0,0)
        a(0,0,1,0)=-fac*epomn*root3*zp(4)
        a(1,1,1,0)=-a(0,0,1,0)
        a(1,0,1,0)=fac*epomn*root3*zp(3)
        a(0,1,1,0)=-a(1,0,1,0)
c
        a(0,0,0,1)=fac*gamma1
        a(1,1,0,1)=a(0,0,0,1)
        a(1,0,0,1)=fac*gamma3
        a(0,1,0,1)=a(1,0,0,1)
        a(0,0,1,1)=-fac*(gamma2+pomn*gamma1)/epomn
        a(1,1,1,1)=-a(0,0,1,1)
        a(1,0,1,1)=-fac*(pomn*gamma3-gamma4)/epomn
        a(0,1,1,1)=-a(1,0,1,1)
c
c        write(6,*) p,a
c
c      else if(irel.eq.0) then          ! nonrelativistic wave functions
c        a(0,0,0)=fac*anrmnr*(zp(1)+root2*zp(2))
c        a(1,1,0)=a(0,0,0)
c        a(1,0,0)=fac*anrmnr*(root2*zp(1)-zp(2))
c        a(0,1,0)=a(1,0,0)
c        a(0,0,1)=0.0
c        a(1,1,1)=0.0
c        a(1,0,1)=0.0
c        a(0,1,1)=0.0
c      end if
c
      return
      end
c
c______________________________________________________________________
c
      SUBROUTINE DEEP_SPLINE(X,Y,N,m,np,mp,YP1,YPN,Y2)
c
      implicit real*8(a-h,o-z)
c
      PARAMETER (NMAX=100)
c      DIMENSION X(N),Y(N,m),Y2(N,m),U(NMAX)
      DIMENSION X(np),Y(np,mp),Y2(np,mp),U(NMAX)
c
      do 100 j=1,m
c
        IF (YP1.GT..99E30) THEN
          Y2(1,j)=0.
          U(1)=0.
        ELSE
          Y2(1,j)=-0.5d0
          U(1)=(3.d0/(X(2)-X(1)))*((Y(2,j)-Y(1,j))/(X(2)-X(1))-YP1)
        ENDIF
        DO 11 I=2,N-1
          SIG=(X(I)-X(I-1))/(X(I+1)-X(I-1))
          P=SIG*Y2(I-1,j)+2.
          Y2(I,j)=(SIG-1.)/P
          U(I)=(6.d0*((Y(I+1,j)-Y(I,j))/(X(I+1)-X(I))-(Y(I,j)-Y(I-1,j))
     *        /(X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*U(I-1))/P
11      CONTINUE
        IF (YPN.GT..99d30) THEN
          QN=0.
          UN=0.
        ELSE
          QN=0.5
          UN=(3.d0/(X(N)-X(N-1)))*(YPN-(Y(N,j)-Y(N-1,j))/(X(N)-X(N-1)))
        ENDIF
        Y2(N,j)=(UN-QN*U(N-1))/(QN*Y2(N-1,j)+1.)
        DO 12 K=N-1,1,-1
          Y2(K,j)=Y2(K,j)*Y2(K+1,j)+U(K)
12      CONTINUE
c
 100  continue
      RETURN
      END
c
c__________________________________________________________________________
c
      SUBROUTINE DEEP_SPLINT(XA,YA,Y2A,N,m,np,mp,X,Y)
c
      implicit real*8(a-h,o-z)
c
      DIMENSION XA(np),YA(np,mp),Y2A(np,mp),y(mp)
      KLO=1
      KHI=N
1     IF (KHI-KLO.GT.1) THEN
        K=(KHI+KLO)/2
        IF(XA(K).GT.X)THEN
          KHI=K
        ELSE
          KLO=K
        ENDIF
      GOTO 1
      ENDIF
      H=XA(KHI)-XA(KLO)
      IF (H.EQ.0.) PAUSE 'Bad XA input.'
      A=(XA(KHI)-X)/H
      B=(X-XA(KLO))/H
c
      do 100 j=1,m
       Y(j)=A*YA(KLO,j)+B*YA(KHI,j)+
     *      ((A**3-A)*Y2A(KLO,j)+(B**3-B)*Y2A(KHI,j))*(H**2)/6.d0
 100  continue
c
      RETURN
      END
c
c__________________________________________________________________________
c
c*****fourier_bessel.f
c
c     This subroutine constructs the spherical bessel transform of the
c     deuteron wave functions to convert from momentum space to coordinate
c     space. This uses a modified version of the Filon's integration
c     technique.
c
      subroutine fourier_bessel(r,rwave)
      
      implicit real*8 (a-h,o-z)
c
      dimension zp(4),zj(4),rwave(4),lwave(4)
c
      parameter(hbarc=197.33,pi=3.141592741012573)
      data lwave/0,2,1,1/
c
      fac=sqrt(2./197.33/pi)
c
      twopi32=sqrt((2.*pi)**3)
c
      pmax=2.e4
      dp=1.
      ip=pmax/dp
c
      rmev=r/hbarc
c
      do i=1,4             !clear wave function output vector.
        rwave(i)=0.
      end do
c
      theta=dp*rmev
c
      if(theta.gt.1.e-5) then
        beta=2.*(theta*(1.+cos(theta)**2)-
     &         2.*sin(theta)*cos(theta))/theta**3
        gamma=4.*(sin(theta)-theta*cos(theta))/theta**3
      else
        beta=2./3.+2.*theta**2/15.+4.*theta**4/105.+2.*theta**6/567.
        gamma=4./3.-2.*theta**2/15.+theta**4/210.-theta**6/11340.
      endif
c
      do j=1,ip             ! integration loop
c
        p=dp*j
        z=p*rmev
        jodd=mod(j,2)
c
        if(jodd.eq.0) then
          qfac=fac*beta
        else
          qfac=fac*gamma
        endif
c
        call wav(p,zp)
c
        if(z.gt.1.e-5) then
          zj(1)=sin(z)
          zj(2)=(3./z**2-1.)*zj(1)-3.*cos(z)/z
          zj(3)=zj(1)/z-cos(z)
        else
          zj(1)=z*(1.-z**2/6.+z**4/280.)
          zj(2)=z**3*(1.-z**2/14.+z**4/504.)/15.
          zj(3)=z**2*(1.-z**2/10.+z**4/280.)/3.
        endif
        zj(4)=zj(3)
c
        do k=1,4
          rwave(k)=rwave(k)+qfac*dp*p*zj(k)*zp(k)/twopi32
        end do
c
      end do               !end integration loop
c
c
      return
      end
